@@ -18,9 +18,12 @@ import hashlib
 import logging
 import math
 import os
+import shlex
 import socket
 import time
 
+from keystoneauth1.access import service_catalog as keystone_sc
+from keystoneauth1 import exceptions as keystone_exc
 from oslo_concurrency import processutils
 from oslo_config import cfg
 from oslo_utils import units
@@ -29,19 +32,19 @@ from glance_store import capabilities
 from glance_store.common import utils
 import glance_store.driver
 from glance_store import exceptions
-from glance_store.i18n import _, _LE, _LW, _LI
+from glance_store.i18n import _, _LE, _LI
 import glance_store.location
-from keystoneclient import exceptions as keystone_exc
-from keystoneclient import service_catalog as keystone_sc
 
 try:
     from cinderclient import exceptions as cinder_exception
     from cinderclient.v2 import client as cinderclient
     from os_brick.initiator import connector
+    from oslo_privsep import priv_context
 except ImportError:
     cinder_exception = None
     cinderclient = None
     connector = None
+    priv_context = None
 
 
 CONF = cfg.CONF
@@ -50,7 +53,7 @@ LOG = logging.getLogger(__name__)
 _CINDER_OPTS = [
     cfg.StrOpt('cinder_catalog_info',
                default='volumev2::publicURL',
-               help=_("""
+               help="""
 Information to match when looking for cinder in the service catalog.
 
 When the ``cinder_endpoint_template`` is not set and any of
@@ -64,8 +67,8 @@ The service catalog can be listed by the ``openstack catalog list`` command.
 
 Possible values:
     * A string of of the following form:
-      ``<service_type>:<service_name>:<endpoint_type>``
-      At least ``service_type`` and ``endpoint_type`` should be specified.
+      ``<service_type>:<service_name>:<interface>``
+      At least ``service_type`` and ``interface`` should be specified.
       ``service_name`` can be omitted.
 
 Related options:
@@ -76,10 +79,10 @@ Related options:
     * cinder_store_project_name
     * cinder_store_password
 
-""")),
+"""),
     cfg.StrOpt('cinder_endpoint_template',
                default=None,
-               help=_("""
+               help="""
 Override service catalog lookup with template for cinder endpoint.
 
 When this option is set, this value is used to generate cinder endpoint,
@@ -102,10 +105,10 @@ Related options:
     * cinder_store_password
     * cinder_catalog_info
 
-""")),
+"""),
     cfg.StrOpt('cinder_os_region_name', deprecated_name='os_region_name',
                default=None,
-               help=_("""
+               help="""
 Region name to lookup cinder service from the service catalog.
 
 This is used only when ``cinder_catalog_info`` is used for determining the
@@ -119,9 +122,9 @@ Possible values:
 Related options:
     * cinder_catalog_info
 
-""")),
+"""),
     cfg.StrOpt('cinder_ca_certificates_file',
-               help=_("""
+               help="""
 Location of a CA certificates file used for cinder client requests.
 
 The specified CA certificates file, if set, is used to verify cinder
@@ -134,11 +137,11 @@ Possible values:
 Related options:
     * cinder_api_insecure
 
-""")),
+"""),
     cfg.IntOpt('cinder_http_retries',
                min=0,
                default=3,
-               help=_("""
+               help="""
 Number of cinderclient retries on failed http calls.
 
 When a call failed by any errors, cinderclient will retry the call up to the
@@ -150,11 +153,11 @@ Possible values:
 Related options:
     * None
 
-""")),
+"""),
     cfg.IntOpt('cinder_state_transition_timeout',
                min=0,
                default=300,
-               help=_("""
+               help="""
 Time period, in seconds, to wait for a cinder volume transition to
 complete.
 
@@ -171,10 +174,10 @@ Possible values:
 Related options:
     * None
 
-""")),
+"""),
     cfg.BoolOpt('cinder_api_insecure',
                 default=False,
-                help=_("""
+                help="""
 Allow to perform insecure SSL requests to cinder.
 
 If this option is set to True, HTTPS endpoint connection is verified using the
@@ -187,10 +190,10 @@ Possible values:
 Related options:
     * cinder_ca_certificates_file
 
-""")),
+"""),
     cfg.StrOpt('cinder_store_auth_address',
                default=None,
-               help=_("""
+               help="""
 The address where the cinder authentication service is listening.
 
 When all of ``cinder_store_auth_address``, ``cinder_store_user_name``,
@@ -212,10 +215,10 @@ Related options:
     * cinder_store_password
     * cinder_store_project_name
 
-""")),
+"""),
     cfg.StrOpt('cinder_store_user_name',
                default=None,
-               help=_("""
+               help="""
 User name to authenticate against cinder.
 
 This must be used with all the following related options. If any of these are
@@ -229,9 +232,9 @@ Related options:
     * cinder_store_password
     * cinder_store_project_name
 
-""")),
+"""),
     cfg.StrOpt('cinder_store_password', secret=True,
-               help=_("""
+               help="""
 Password for the user authenticating against cinder.
 
 This must be used with all the following related options. If any of these are
@@ -245,10 +248,10 @@ Related options:
     * cinder_store_user_name
     * cinder_store_project_name
 
-""")),
+"""),
     cfg.StrOpt('cinder_store_project_name',
                default=None,
-               help=_("""
+               help="""
 Project name where the image volume is stored in cinder.
 
 If this configuration option is not set, the project in current context is
@@ -265,10 +268,10 @@ Related options:
     * ``cinder_store_user_name``
     * ``cinder_store_password``
 
-""")),
+"""),
     cfg.StrOpt('rootwrap_config',
                default='/etc/glance/rootwrap.conf',
-               help=_("""
+               help="""
 Path to the rootwrap configuration file to use for running commands as root.
 
 The cinder store requires root privileges to operate the image volumes (for
@@ -282,23 +285,56 @@ Possible values:
 Related options:
     * None
 
-""")),
+"""),
+    cfg.StrOpt('cinder_volume_type',
+               default=None,
+               help="""
+Volume type that will be used for volume creation in cinder.
+
+Some cinder backends can have several volume types to optimize storage usage.
+Adding this option allows an operator to choose a specific volume type
+in cinder that can be optimized for images.
+
+If this is not set, then the default volume type specified in the cinder
+configuration will be used for volume creation.
+
+Possible values:
+    * A valid volume type from cinder
+
+Related options:
+    * None
+
+"""),
 ]
 
 
-def get_root_helper():
-    return 'sudo glance-rootwrap %s' % CONF.glance_store.rootwrap_config
+def get_root_helper(backend=None):
+    if backend:
+        rootwrap = getattr(CONF, backend).rootwrap_config
+    else:
+        rootwrap = CONF.glance_store.rootwrap_config
+
+    return 'sudo glance-rootwrap %s' % rootwrap
 
 
-def is_user_overriden(conf):
-    return all([conf.glance_store.get('cinder_store_' + key)
+def is_user_overriden(conf, backend=None):
+    if backend:
+        store_conf = getattr(conf, backend)
+    else:
+        store_conf = conf.glance_store
+
+    return all([store_conf.get('cinder_store_' + key)
                 for key in ['user_name', 'password',
                             'project_name', 'auth_address']])
 
 
-def get_cinderclient(conf, context=None):
-    glance_store = conf.glance_store
-    user_overriden = is_user_overriden(conf)
+def get_cinderclient(conf, context=None, backend=None):
+    if backend:
+        glance_store = getattr(conf, backend)
+    else:
+        glance_store = conf.glance_store
+
+    user_overriden = is_user_overriden(conf, backend=backend)
     if user_overriden:
         username = glance_store.cinder_store_user_name
         password = glance_store.cinder_store_password
@@ -313,14 +349,14 @@ def get_cinderclient(conf, context=None):
             url = glance_store.cinder_endpoint_template % context.to_dict()
         else:
             info = glance_store.cinder_catalog_info
-            service_type, service_name, endpoint_type = info.split(':')
-            sc = {'serviceCatalog': context.service_catalog}
+            service_type, service_name, interface = info.split(':')
             try:
-                url = keystone_sc.ServiceCatalogV2(sc).url_for(
+                catalog = keystone_sc.ServiceCatalogV2(context.service_catalog)
+                url = catalog.url_for(
                     region_name=glance_store.cinder_os_region_name,
                     service_type=service_type,
                     service_name=service_name,
-                    endpoint_type=endpoint_type)
+                    interface=interface)
             except keystone_exc.EndpointNotFound:
                 reason = _("Failed to find Cinder from a service catalog.")
                 raise exceptions.BadStoreConfiguration(store_name="cinder",
@@ -330,6 +366,7 @@ def get_cinderclient(conf, context=None):
                             password,
                             project,
                             auth_url=url,
+                            region_name=glance_store.cinder_os_region_name,
                             insecure=glance_store.cinder_api_insecure,
                             retries=glance_store.cinder_http_retries,
                             cacert=glance_store.cinder_ca_certificates_file)
@@ -357,10 +394,7 @@ class StoreLocation(glance_store.location.StoreLocation):
         return "cinder://%s" % self.volume_id
 
     def parse_uri(self, uri):
-        if not uri.startswith('cinder://'):
-            reason = _("URI must start with 'cinder://'")
-            LOG.info(reason)
-            raise exceptions.BadStoreUri(message=reason)
+        self.validate_schemas(uri, valid_schemas=('cinder://',))
 
         self.scheme = 'cinder'
         self.volume_id = uri[9:]
@@ -372,21 +406,21 @@ class StoreLocation(glance_store.location.StoreLocation):
 
 
 @contextlib.contextmanager
-def temporary_chown(path):
+def temporary_chown(path, backend=None):
     owner_uid = os.getuid()
     orig_uid = os.stat(path).st_uid
 
     if orig_uid != owner_uid:
         processutils.execute('chown', owner_uid, path,
                              run_as_root=True,
-                             root_helper=get_root_helper())
+                             root_helper=get_root_helper(backend=backend))
     try:
         yield
     finally:
         if orig_uid != owner_uid:
             processutils.execute('chown', orig_uid, path,
                                  run_as_root=True,
-                                 root_helper=get_root_helper())
+                                 root_helper=get_root_helper(backend=backend))
 
 
 class Store(glance_store.driver.Store):
@@ -401,15 +435,13 @@ class Store(glance_store.driver.Store):
 
     def __init__(self, *args, **kargs):
         super(Store, self).__init__(*args, **kargs)
-        LOG.warning(_LW("Cinder store is considered experimental. "
-                        "Current deployers should be aware that the use "
-                        "of it in production right now may be risky."))
 
     def get_schemes(self):
         return ('cinder',)
 
     def _check_context(self, context, require_tenant=False):
-        user_overriden = is_user_overriden(self.conf)
+        user_overriden = is_user_overriden(self.conf,
+                                           backend=self.backend_group)
         if user_overriden and not require_tenant:
             return
         if context is None:
@@ -423,7 +455,12 @@ class Store(glance_store.driver.Store):
 
     def _wait_volume_status(self, volume, status_transition, status_expected):
         max_recheck_wait = 15
-        timeout = self.conf.glance_store.cinder_state_transition_timeout
+        if self.backend_group:
+            timeout = getattr(
+                self.conf, self.backend_group).cinder_state_transition_timeout
+        else:
+            timeout = self.conf.glance_store.cinder_state_transition_timeout
+
         volume = volume.manager.get(volume.id)
         tries = 0
         elapsed = 0
@@ -453,7 +490,8 @@ class Store(glance_store.driver.Store):
     def _open_cinder_volume(self, client, volume, mode):
         attach_mode = 'rw' if mode == 'wb' else 'ro'
         device = None
-        root_helper = get_root_helper()
+        root_helper = get_root_helper(backend=self.backend_group)
+        priv_context.init(root_helper=shlex.split(root_helper))
         host = socket.gethostname()
         properties = connector.get_connector_properties(root_helper, host,
                                                         False, False)
@@ -472,12 +510,16 @@ class Store(glance_store.driver.Store):
                 connection_info['driver_volume_type'], root_helper,
                 conn=connection_info)
             device = conn.connect_volume(connection_info['data'])
-            volume.attach(None, None, attach_mode, host_name=host)
+            volume.attach(None, 'glance_store', attach_mode, host_name=host)
             volume = self._wait_volume_status(volume, 'attaching', 'in-use')
-            LOG.debug('Opening host device "%s"', device['path'])
-            with temporary_chown(device['path']), \
-                    open(device['path'], mode) as f:
-                yield f
+            if (connection_info['driver_volume_type'] == 'rbd' and
+               not conn.do_local_attach):
+                yield device['path']
+            else:
+                with temporary_chown(device['path'],
+                                     backend=self.backend_group), \
+                        open(device['path'], mode) as f:
+                    yield f
         except Exception:
             LOG.exception(_LE('Exception while accessing to cinder volume '
                               '%(volume_id)s.'), {'volume_id': volume.id})
@@ -542,18 +584,19 @@ class Store(glance_store.driver.Store):
         where to find the image file, and returns a tuple of generator
         (for reading the image file) and image_size
 
-        :param location `glance_store.location.Location` object, supplied
+        :param location: `glance_store.location.Location` object, supplied
                         from glance_store.location.get_location_from_uri()
         :param offset: offset to start reading
         :param chunk_size: size to read, or None to get all the image
         :param context: Request context
-        :raises `glance_store.exceptions.NotFound` if image does not exist
+        :raises: `glance_store.exceptions.NotFound` if image does not exist
         """
 
         loc = location.store_location
         self._check_context(context)
         try:
-            client = get_cinderclient(self.conf, context)
+            client = get_cinderclient(self.conf, context,
+                                      backend=self.backend_group)
             volume = client.volumes.get(loc.volume_id)
             size = int(volume.metadata.get('image_size',
                                            volume.size * units.Gi))
@@ -563,11 +606,11 @@ class Store(glance_store.driver.Store):
             return (iterator, chunk_size or size)
         except cinder_exception.NotFound:
             reason = _("Failed to get image size due to "
-                       "volume can not be found: %s") % volume.id
+                       "volume can not be found: %s") % loc.volume_id
             LOG.error(reason)
             raise exceptions.NotFound(reason)
         except cinder_exception.ClientException as e:
-            msg = (_('Failed to get image volume %(volume_id): %(error)s')
+            msg = (_('Failed to get image volume %(volume_id)s: %(error)s')
                    % {'volume_id': loc.volume_id, 'error': e})
             LOG.error(msg)
             raise exceptions.BackendException(msg)
@@ -580,15 +623,16 @@ class Store(glance_store.driver.Store):
         :param location: `glance_store.location.Location` object, supplied
                         from glance_store.location.get_location_from_uri()
         :raises: `glance_store.exceptions.NotFound` if image does not exist
-        :rtype int
+        :rtype: int
         """
 
         loc = location.store_location
 
         try:
             self._check_context(context)
-            volume = get_cinderclient(self.conf,
-                                      context).volumes.get(loc.volume_id)
+            volume = get_cinderclient(
+                self.conf, context,
+                backend=self.backend_group).volumes.get(loc.volume_id)
             return int(volume.metadata.get('image_size',
                                            volume.size * units.Gi))
         except cinder_exception.NotFound:
@@ -598,8 +642,9 @@ class Store(glance_store.driver.Store):
                               "internal error."))
             return 0
 
+    @glance_store.driver.back_compat_add
     @capabilities.check
-    def add(self, image_id, image_file, image_size, context=None,
+    def add(self, image_id, image_file, image_size, hashing_algo, context=None,
             verifier=None):
         """
         Stores an image file with supplied identifier to the backend
@@ -609,18 +654,21 @@ class Store(glance_store.driver.Store):
         :param image_id: The opaque image identifier
         :param image_file: The image data to write, as a file-like object
         :param image_size: The size of the image data to write, in bytes
+        :param hashing_algo: A hashlib algorithm identifier (string)
         :param context: The request context
         :param verifier: An object used to verify signatures for images
 
-        :retval tuple of URL in backing store, bytes written, checksum
-                and a dictionary with storage system specific information
-        :raises `glance_store.exceptions.Duplicate` if the image already
-                existed
+        :returns: tuple of: (1) URL in backing store, (2) bytes written,
+                  (3) checksum, (4) multihash value, and (5) a dictionary
+                  with storage system specific information
+        :raises: `glance_store.exceptions.Duplicate` if the image already
+                 exists
         """
 
         self._check_context(context, require_tenant=True)
-        client = get_cinderclient(self.conf, context)
-
+        client = get_cinderclient(self.conf, context,
+                                  backend=self.backend_group)
+        os_hash_value = hashlib.new(str(hashing_algo))
         checksum = hashlib.md5()
         bytes_written = 0
         size_gb = int(math.ceil(float(image_size) / units.Gi))
@@ -631,14 +679,23 @@ class Store(glance_store.driver.Store):
         metadata = {'glance_image_id': image_id,
                     'image_size': str(image_size),
                     'image_owner': owner}
-        LOG.debug('Creating a new volume: image_size=%d size_gb=%d',
-                  image_size, size_gb)
+
+        if self.backend_group:
+            volume_type = getattr(self.conf,
+                                  self.backend_group).cinder_volume_type
+        else:
+            volume_type = self.conf.glance_store.cinder_volume_type
+
+        LOG.debug('Creating a new volume: image_size=%d size_gb=%d type=%s',
+                  image_size, size_gb, volume_type or 'None')
         if image_size == 0:
             LOG.info(_LI("Since image size is zero, we will be doing "
                          "resize-before-write for each GB which "
                          "will be considerably slower than normal."))
-        volume = client.volumes.create(size_gb, name=name, metadata=metadata)
+        volume = client.volumes.create(size_gb, name=name, metadata=metadata,
+                                       volume_type=volume_type)
         volume = self._wait_volume_status(volume, 'creating', 'available')
+        size_gb = volume.size
 
         failed = True
         need_extend = True
@@ -655,6 +712,7 @@ class Store(glance_store.driver.Store):
                         if not buf:
                             need_extend = False
                             break
+                        os_hash_value.update(buf)
                         checksum.update(buf)
                         if verifier:
                             verifier.update(buf)
@@ -673,6 +731,7 @@ class Store(glance_store.driver.Store):
                         volume = self._wait_volume_status(volume,
                                                           'extending',
                                                           'available')
+                        size_gb = volume.size
                     except exceptions.BackendException:
                         raise exceptions.StorageFull()
 
@@ -699,6 +758,7 @@ class Store(glance_store.driver.Store):
             volume.update_all_metadata(metadata)
         volume.update_readonly_flag(volume, True)
 
+        hash_hex = os_hash_value.hexdigest()
         checksum_hex = checksum.hexdigest()
 
         LOG.debug("Wrote %(bytes_written)d bytes to volume %(volume_id)s "
@@ -707,7 +767,15 @@ class Store(glance_store.driver.Store):
                    'volume_id': volume.id,
                    'checksum_hex': checksum_hex})
 
-        return ('cinder://%s' % volume.id, bytes_written, checksum_hex, {})
+        image_metadata = {}
+        if self.backend_group:
+            image_metadata['backend'] = u"%s" % self.backend_group
+
+        return ('cinder://%s' % volume.id,
+                bytes_written,
+                checksum_hex,
+                hash_hex,
+                image_metadata)
 
     @capabilities.check
     def delete(self, location, context=None):
@@ -715,17 +783,18 @@ class Store(glance_store.driver.Store):
         Takes a `glance_store.location.Location` object that indicates
         where to find the image file to delete
 
-        :location `glance_store.location.Location` object, supplied
+        :param location: `glance_store.location.Location` object, supplied
                   from glance_store.location.get_location_from_uri()
 
-        :raises NotFound if image does not exist
-        :raises Forbidden if cannot delete because of permissions
+        :raises: NotFound if image does not exist
+        :raises: Forbidden if cannot delete because of permissions
         """
         loc = location.store_location
         self._check_context(context)
         try:
-            volume = get_cinderclient(self.conf,
-                                      context).volumes.get(loc.volume_id)
+            volume = get_cinderclient(
+                self.conf, context,
+                backend=self.backend_group).volumes.get(loc.volume_id)
             volume.delete()
         except cinder_exception.NotFound:
             raise exceptions.NotFound(image=loc.volume_id)
