@@ -25,7 +25,7 @@ import requests
 from glance_store import capabilities
 import glance_store.driver
 from glance_store import exceptions
-from glance_store.i18n import _
+from glance_store.i18n import _, _LI
 import glance_store.location
 
 LOG = logging.getLogger(__name__)
@@ -35,23 +35,64 @@ MAX_REDIRECTS = 5
 
 _HTTP_OPTS = [
     cfg.StrOpt('https_ca_certificates_file',
-               help=_('Specify the path to the CA bundle file to use in '
-                      'verifying the remote server certificate.')),
+               help="""
+Path to the CA bundle file.
+
+This configuration option enables the operator to use a custom
+Certificate Authority file to verify the remote server certificate. If
+this option is set, the ``https_insecure`` option will be ignored and
+the CA file specified will be used to authenticate the server
+certificate and establish a secure connection to the server.
+
+Possible values:
+    * A valid path to a CA file
+
+Related options:
+    * https_insecure
+
+"""),
     cfg.BoolOpt('https_insecure',
                 default=True,
-                help=_('If true, the remote server certificate is not '
-                       'verified. If false, then the default CA truststore is '
-                       'used for verification. This option is ignored if '
-                       '"https_ca_certificates_file" is set.')),
+                help="""
+Set verification of the remote server certificate.
+
+This configuration option takes in a boolean value to determine
+whether or not to verify the remote server certificate. If set to
+True, the remote server certificate is not verified. If the option is
+set to False, then the default CA truststore is used for verification.
+
+This option is ignored if ``https_ca_certificates_file`` is set.
+The remote server certificate will then be verified using the file
+specified using the ``https_ca_certificates_file`` option.
+
+Possible values:
+    * True
+    * False
+
+Related options:
+    * https_ca_certificates_file
+
+"""),
     cfg.DictOpt('http_proxy_information',
                 default={},
-                help=_('Specify the http/https proxy information that should '
-                       'be used to connect to the remote server. The proxy '
-                       'information should be a key value pair of the '
-                       'scheme and proxy. e.g. http:10.0.0.1:3128. You can '
-                       'specify proxies for multiple schemes by seperating '
-                       'the key value pairs with a comma.'
-                       'e.g. http:10.0.0.1:3128, https:10.0.0.1:1080.'))]
+                help="""
+The http/https proxy information to be used to connect to the remote
+server.
+
+This configuration option specifies the http/https proxy information
+that should be used to connect to the remote server. The proxy
+information should be a key value pair of the scheme and proxy, for
+example, http:10.0.0.1:3128. You can also specify proxies for multiple
+schemes by separating the key value pairs with a comma, for example,
+http:10.0.0.1:3128, https:10.0.0.1:1080.
+
+Possible values:
+    * A comma separated list of scheme:proxy pairs as described above
+
+Related options:
+    * None
+
+""")]
 
 
 class StoreLocation(glance_store.location.StoreLocation):
@@ -84,7 +125,7 @@ class StoreLocation(glance_store.location.StoreLocation):
         versions of Python.
         """
         pieces = urllib.parse.urlparse(uri)
-        assert pieces.scheme in ('https', 'http')
+        self.validate_schemas(uri, valid_schemas=('https://', 'http://'))
         self.scheme = pieces.scheme
         netloc = pieces.netloc
         path = pieces.path
@@ -110,7 +151,7 @@ class StoreLocation(glance_store.location.StoreLocation):
         else:
             self.user = None
         if netloc == '':
-            LOG.info(_("No address specified in HTTP URL"))
+            LOG.info(_LI("No address specified in HTTP URL"))
             raise exceptions.BadStoreUri(uri=uri)
         else:
             # IPv6 address has the following format [1223:0:0:..]:<some_port>
@@ -246,7 +287,8 @@ class Store(glance_store.driver.Store):
                                               self.conf,
                                               uri=url,
                                               image_id=image_id,
-                                              store_specs=store_specs)
+                                              store_specs=store_specs,
+                                              backend=self.backend_group)
 
     @staticmethod
     def _check_store_uri(conn, loc):
@@ -268,7 +310,7 @@ class Store(glance_store.driver.Store):
 
         if conn.is_redirect and conn.status_code not in (301, 302):
             reason = (_("The HTTP URL %(url)s attempted to redirect "
-                        "with an invalid %(status)s status code.") %
+                        "with an invalid %(status)s status code."),
                       {'url': loc.path, 'status': conn.status_code})
             LOG.info(reason)
             raise exceptions.BadStoreUri(message=reason)
@@ -276,9 +318,15 @@ class Store(glance_store.driver.Store):
     def _get_response(self, location, verb):
         if not hasattr(self, 'session'):
             self.session = requests.Session()
-        ca_bundle = self.conf.glance_store.https_ca_certificates_file
-        disable_https = self.conf.glance_store.https_insecure
+
+        if self.backend_group:
+            store_conf = getattr(self.conf, self.backend_group)
+        else:
+            store_conf = self.conf.glance_store
+
+        ca_bundle = store_conf.https_ca_certificates_file
+        disable_https = store_conf.https_insecure
         self.session.verify = ca_bundle if ca_bundle else not disable_https
-        self.session.proxies = self.conf.glance_store.http_proxy_information
+        self.session.proxies = store_conf.http_proxy_information
         return self.session.request(verb, location.get_uri(), stream=True,
                                     allow_redirects=False)
