@@ -25,7 +25,6 @@ import math
 
 from oslo_config import cfg
 from oslo_utils import units
-import six
 from six.moves import urllib
 
 from glance_store import capabilities
@@ -53,7 +52,7 @@ LOG = logging.getLogger(__name__)
 _RBD_OPTS = [
     cfg.IntOpt('rbd_store_chunk_size', default=DEFAULT_CHUNKSIZE,
                min=1,
-               help=_("""
+               help="""
 Size, in megabytes, to chunk RADOS images into.
 
 Provide an integer value representing the size in megabytes to chunk
@@ -71,9 +70,9 @@ Possible Values:
 Related options:
     * None
 
-""")),
+"""),
     cfg.StrOpt('rbd_store_pool', default=DEFAULT_POOL,
-               help=_("""
+               help="""
 RADOS pool in which images are stored.
 
 When RBD is used as the storage backend for storing Glance images, the
@@ -91,9 +90,9 @@ Possible Values:
 Related options:
     * None
 
-""")),
+"""),
     cfg.StrOpt('rbd_store_user', default=DEFAULT_USER,
-               help=_("""
+               help="""
 RADOS user to authenticate as.
 
 This configuration option takes in the RADOS user to authenticate as.
@@ -109,9 +108,9 @@ Possible Values:
 Related options:
     * rbd_store_ceph_conf
 
-""")),
+"""),
     cfg.StrOpt('rbd_store_ceph_conf', default=DEFAULT_CONFFILE,
-               help=_("""
+               help="""
 Ceph configuration file path.
 
 This configuration option takes in the path to the Ceph configuration
@@ -127,9 +126,9 @@ Possible Values:
 Related options:
     * rbd_store_user
 
-""")),
+"""),
     cfg.IntOpt('rados_connect_timeout', default=0,
-               help=_("""
+               help="""
 Timeout value for connecting to Ceph cluster.
 
 This configuration option takes in the timeout value in seconds used
@@ -145,7 +144,7 @@ Possible Values:
 Related options:
     * None
 
-"""))
+"""),
 ]
 
 
@@ -162,7 +161,7 @@ class StoreLocation(location.StoreLocation):
 
     def process_specs(self):
         # convert to ascii since librbd doesn't handle unicode
-        for key, value in six.iteritems(self.specs):
+        for key, value in self.specs.items():
             self.specs[key] = str(value)
         self.fsid = self.specs.get('fsid')
         self.pool = self.specs.get('pool')
@@ -183,12 +182,7 @@ class StoreLocation(location.StoreLocation):
 
     def parse_uri(self, uri):
         prefix = 'rbd://'
-        if not uri.startswith(prefix):
-            reason = _('URI must start with rbd://')
-            msg = _LI("Invalid URI: %s") % reason
-
-            LOG.info(msg)
-            raise exceptions.BadStoreUri(message=reason)
+        self.validate_schemas(uri, valid_schemas=(prefix,))
         # convert to ascii since librbd doesn't handle unicode
         try:
             ascii_uri = str(uri)
@@ -240,8 +234,7 @@ class ImageIterator(object):
                 with conn.open_ioctx(self.pool) as ioctx:
                     with rbd.Image(ioctx, self.name,
                                    snapshot=self.snapshot) as image:
-                        img_info = image.stat()
-                        size = img_info['size']
+                        size = image.size()
                         bytes_left = size
                         while bytes_left > 0:
                             length = min(self.chunk_size, bytes_left)
@@ -288,17 +281,32 @@ class Store(driver.Store):
         itself, it should raise `exceptions.BadStoreConfiguration`
         """
         try:
-            chunk = self.conf.glance_store.rbd_store_chunk_size
+            if self.backend_group:
+                chunk = getattr(self.conf,
+                                self.backend_group).rbd_store_chunk_size
+                pool = getattr(self.conf, self.backend_group).rbd_store_pool
+                user = getattr(self.conf, self.backend_group).rbd_store_user
+                conf_file = getattr(self.conf,
+                                    self.backend_group).rbd_store_ceph_conf
+                connect_timeout = getattr(
+                    self.conf, self.backend_group).rados_connect_timeout
+            else:
+                chunk = self.conf.glance_store.rbd_store_chunk_size
+                pool = self.conf.glance_store.rbd_store_pool
+                user = self.conf.glance_store.rbd_store_user
+                conf_file = self.conf.glance_store.rbd_store_ceph_conf
+                connect_timeout = self.conf.glance_store.rados_connect_timeout
+
             self.chunk_size = chunk * units.Mi
             self.READ_CHUNKSIZE = self.chunk_size
             self.WRITE_CHUNKSIZE = self.READ_CHUNKSIZE
 
             # these must not be unicode since they will be passed to a
             # non-unicode-aware C library
-            self.pool = str(self.conf.glance_store.rbd_store_pool)
-            self.user = str(self.conf.glance_store.rbd_store_user)
-            self.conf_file = str(self.conf.glance_store.rbd_store_ceph_conf)
-            self.connect_timeout = self.conf.glance_store.rados_connect_timeout
+            self.pool = str(pool)
+            self.user = str(user)
+            self.conf_file = str(conf_file)
+            self.connect_timeout = connect_timeout
         except cfg.ConfigFileValueError as e:
             reason = _("Error in store configuration: %s") % e
             LOG.error(reason)
@@ -355,7 +363,7 @@ class Store(driver.Store):
 
         :param image_name: Image's name
 
-        :retval: `glance_store.rbd.StoreLocation` object
+        :returns: `glance_store.rbd.StoreLocation` object
         """
         librbd = rbd.RBD()
         features = conn.conf_get('rbd_default_features')
@@ -389,7 +397,7 @@ class Store(driver.Store):
                     if snapshot_name is not None:
                         with rbd.Image(ioctx, image_name) as image:
                             try:
-                                image.unprotect_snap(snapshot_name)
+                                self._unprotect_snapshot(image, snapshot_name)
                                 image.remove_snap(snapshot_name)
                             except rbd.ImageNotFound as exc:
                                 msg = (_("Snap Operating Exception "
@@ -423,8 +431,18 @@ class Store(driver.Store):
                     msg = _("RBD image %s does not exist") % image_name
                     raise exceptions.NotFound(message=msg)
 
+    def _unprotect_snapshot(self, image, snap_name):
+        try:
+            image.unprotect_snap(snap_name)
+        except rbd.InvalidArgument:
+            # NOTE(slaweq): if snapshot was unprotected already, rbd library
+            # raises InvalidArgument exception without any "clear" message.
+            # Such exception is not dangerous for us so it will be just logged
+            LOG.debug("Snapshot %s is unprotected already" % snap_name)
+
+    @driver.back_compat_add
     @capabilities.check
-    def add(self, image_id, image_file, image_size, context=None,
+    def add(self, image_id, image_file, image_size, hashing_algo, context=None,
             verifier=None):
         """
         Stores an image file with supplied identifier to the backend
@@ -434,14 +452,18 @@ class Store(driver.Store):
         :param image_id: The opaque image identifier
         :param image_file: The image data to write, as a file-like object
         :param image_size: The size of the image data to write, in bytes
+        :param hashing_algo: A hashlib algorithm identifier (string)
+        :param context: A context object
         :param verifier: An object used to verify signatures for images
 
-        :retval: tuple of URL in backing store, bytes written, checksum
-                and a dictionary with storage system specific information
+        :returns: tuple of: (1) URL in backing store, (2) bytes written,
+                  (3) checksum, (4) multihash value, and (5) a dictionary
+                  with storage system specific information
         :raises: `glance_store.exceptions.Duplicate` if the image already
-                existed
+                 exists
         """
         checksum = hashlib.md5()
+        os_hash_value = hashlib.new(str(hashing_algo))
         image_name = str(image_id)
         with self.get_connection(conffile=self.conf_file,
                                  rados_id=self.user) as conn:
@@ -485,6 +507,7 @@ class Store(driver.Store):
                             LOG.debug(_("writing chunk at offset %s") %
                                       (offset))
                             offset += image.write(chunk, offset)
+                            os_hash_value.update(chunk)
                             checksum.update(chunk)
                             if verifier:
                                 verifier.update(chunk)
@@ -512,7 +535,16 @@ class Store(driver.Store):
         if image_size == 0:
             image_size = bytes_written
 
-        return (loc.get_uri(), image_size, checksum.hexdigest(), {})
+        # Add store backend information to location metadata
+        metadata = {}
+        if self.backend_group:
+            metadata['backend'] = u"%s" % self.backend_group
+
+        return (loc.get_uri(),
+                image_size,
+                checksum.hexdigest(),
+                os_hash_value.hexdigest(),
+                metadata)
 
     @capabilities.check
     def delete(self, location, context=None):
