@@ -44,7 +44,7 @@ _SHEEPDOG_OPTS = [
     cfg.IntOpt('sheepdog_store_chunk_size',
                min=1,
                default=DEFAULT_CHUNKSIZE,
-               help=_("""
+               help="""
 Chunk size for images to be stored in Sheepdog data store.
 
 Provide an integer value representing the size in mebibyte
@@ -64,10 +64,10 @@ Possible values:
 Related Options:
     * None
 
-""")),
+"""),
     cfg.PortOpt('sheepdog_store_port',
                 default=DEFAULT_PORT,
-                help=_("""
+                help="""
 Port number on which the sheep daemon will listen.
 
 Provide an integer value representing a valid port number on
@@ -86,10 +86,10 @@ Possible values:
 Related Options:
     * sheepdog_store_address
 
-""")),
-    cfg.StrOpt('sheepdog_store_address',
-               default=DEFAULT_ADDR,
-               help=_("""
+"""),
+    cfg.HostAddressOpt('sheepdog_store_address',
+                       default=DEFAULT_ADDR,
+                       help="""
 Address to bind the Sheepdog daemon to.
 
 Provide a string value representing the address to bind the
@@ -110,7 +110,7 @@ Possible values:
 Related Options:
     * sheepdog_store_port
 
-"""))
+"""),
 ]
 
 
@@ -124,17 +124,14 @@ class SheepdogImage(object):
         self.chunk_size = chunk_size
 
     def _run_command(self, command, data, *params):
-        cmd = ("collie vdi %(command)s -a %(addr)s -p %(port)d %(name)s "
-               "%(params)s" %
-               {"command": command,
-                "addr": self.addr,
-                "port": self.port,
-                "name": self.name,
-                "params": " ".join(map(str, params))})
+        cmd = ['collie', 'vdi']
+        cmd.extend(command.split(' '))
+        cmd.extend(['-a', self.addr, '-p', self.port, self.name])
+        cmd.extend(params)
 
         try:
             return processutils.execute(
-                cmd, process_input=data)[0]
+                *cmd, process_input=data)[0]
         except processutils.ProcessExecutionError as exc:
             LOG.error(exc)
             raise glance_store.BackendException(exc)
@@ -225,9 +222,7 @@ class StoreLocation(glance_store.location.StoreLocation):
 
     def parse_uri(self, uri):
         valid_schema = 'sheepdog://'
-        if not uri.startswith(valid_schema):
-            reason = _("URI must start with '%s'") % valid_schema
-            raise exceptions.BadStoreUri(message=reason)
+        self.validate_schemas(uri, valid_schemas=(valid_schema,))
         pieces = uri[len(valid_schema):].split(':')
         if len(pieces) == 3:
             self.image = pieces[2]
@@ -235,9 +230,14 @@ class StoreLocation(glance_store.location.StoreLocation):
             self.addr = pieces[0]
         # This is used for backwards compatibility.
         else:
+            if self.backend_group:
+                store_conf = getattr(self.conf, self.backend_group)
+            else:
+                store_conf = self.conf.glance_store
+
             self.image = pieces[0]
-            self.port = self.conf.glance_store.sheepdog_store_port
-            self.addr = self.conf.glance_store.sheepdog_store_address
+            self.port = store_conf.sheepdog_store_port
+            self.addr = store_conf.sheepdog_store_address
 
 
 class ImageIterator(object):
@@ -277,15 +277,19 @@ class Store(glance_store.driver.Store):
         this method. If the store was not able to successfully configure
         itself, it should raise `exceptions.BadStoreConfiguration`
         """
+        if self.backend_group:
+            store_conf = getattr(self.conf, self.backend_group)
+        else:
+            store_conf = self.conf.glance_store
 
         try:
-            chunk_size = self.conf.glance_store.sheepdog_store_chunk_size
+            chunk_size = store_conf.sheepdog_store_chunk_size
             self.chunk_size = chunk_size * units.Mi
             self.READ_CHUNKSIZE = self.chunk_size
             self.WRITE_CHUNKSIZE = self.READ_CHUNKSIZE
 
-            self.addr = self.conf.glance_store.sheepdog_store_address
-            self.port = self.conf.glance_store.sheepdog_store_port
+            self.addr = store_conf.sheepdog_store_address
+            self.port = store_conf.sheepdog_store_port
         except cfg.ConfigFileValueError as e:
             reason = _("Error in store configuration: %s") % e
             LOG.error(reason)
@@ -328,7 +332,7 @@ class Store(glance_store.driver.Store):
         :param location: `glance_store.location.Location` object, supplied
                         from glance_store.location.get_location_from_uri()
         :raises: `glance_store.exceptions.NotFound` if image does not exist
-        :param rtype: int
+        :rtype: int
         """
 
         loc = location.store_location
@@ -339,8 +343,9 @@ class Store(glance_store.driver.Store):
                                       % image.name)
         return image.get_size()
 
+    @glance_store.driver.back_compat_add
     @capabilities.check
-    def add(self, image_id, image_file, image_size, context=None,
+    def add(self, image_id, image_file, image_size, hashing_algo, context=None,
             verifier=None):
         """
         Stores an image file with supplied identifier to the backend
@@ -350,11 +355,15 @@ class Store(glance_store.driver.Store):
         :param image_id: The opaque image identifier
         :param image_file: The image data to write, as a file-like object
         :param image_size: The size of the image data to write, in bytes
+        :param hashing_algo: A hashlib algorithm identifier (string)
+        :param context: A context object
         :param verifier: An object used to verify signatures for images
 
-        :retval: tuple of URL in backing store, bytes written, and checksum
+        :returns: tuple of: (1) URL in backing store, (2) bytes written,
+                  (3) checksum, (4) multihash value, and (5) a dictionary
+                  with storage system specific information
         :raises: `glance_store.exceptions.Duplicate` if the image already
-                existed
+                 exists
         """
 
         image = SheepdogImage(self.addr, self.port, image_id,
@@ -367,12 +376,13 @@ class Store(glance_store.driver.Store):
             'image': image_id,
             'addr': self.addr,
             'port': self.port
-        }, self.conf)
+        }, self.conf, backend_group=self.backend_group)
 
         image.create(image_size)
 
         try:
             offset = 0
+            os_hash_value = hashlib.new(str(hashing_algo))
             checksum = hashlib.md5()
             chunks = utils.chunkreadable(image_file, self.WRITE_CHUNKSIZE)
             for chunk in chunks:
@@ -385,6 +395,7 @@ class Store(glance_store.driver.Store):
                     image.resize(offset + chunk_length)
                 image.write(chunk, offset, chunk_length)
                 offset += chunk_length
+                os_hash_value.update(chunk)
                 checksum.update(chunk)
                 if verifier:
                     verifier.update(chunk)
@@ -394,7 +405,15 @@ class Store(glance_store.driver.Store):
             with excutils.save_and_reraise_exception():
                 image.delete()
 
-        return (location.get_uri(), offset, checksum.hexdigest(), {})
+        metadata = {}
+        if self.backend_group:
+            metadata['backend'] = u"%s" % self.backend_group
+
+        return (location.get_uri(),
+                offset,
+                checksum.hexdigest(),
+                os_hash_value.hexdigest(),
+                metadata)
 
     @capabilities.check
     def delete(self, location, context=None):
