@@ -46,8 +46,7 @@ import glance_store
 from glance_store import capabilities
 from glance_store.common import utils
 from glance_store import exceptions
-from glance_store.i18n import _
-from glance_store.i18n import _LE
+from glance_store.i18n import _, _LE
 from glance_store import location
 
 
@@ -59,59 +58,186 @@ DEFAULT_STORE_IMAGE_DIR = '/openstack_glance'
 DS_URL_PREFIX = '/folder'
 STORE_SCHEME = 'vsphere'
 
-# check that datacenter/datastore combination is valid
-_datastore_info_valid = False
-
 _VMWARE_OPTS = [
-    cfg.StrOpt('vmware_server_host',
-               help=_('ESX/ESXi or vCenter Server target system. '
-                      'The server value can be an IP address or a DNS name.')),
+    cfg.HostAddressOpt('vmware_server_host',
+                       sample_default='127.0.0.1',
+                       help="""
+Address of the ESX/ESXi or vCenter Server target system.
+
+This configuration option sets the address of the ESX/ESXi or vCenter
+Server target system. This option is required when using the VMware
+storage backend. The address can contain an IP address (127.0.0.1) or
+a DNS name (www.my-domain.com).
+
+Possible Values:
+    * A valid IPv4 or IPv6 address
+    * A valid DNS name
+
+Related options:
+    * vmware_server_username
+    * vmware_server_password
+
+"""),
     cfg.StrOpt('vmware_server_username',
-               help=_('Username for authenticating with '
-                      'VMware ESX/VC server.')),
+               sample_default='root',
+               help="""
+Server username.
+
+This configuration option takes the username for authenticating with
+the VMware ESX/ESXi or vCenter Server. This option is required when
+using the VMware storage backend.
+
+Possible Values:
+    * Any string that is the username for a user with appropriate
+      privileges
+
+Related options:
+    * vmware_server_host
+    * vmware_server_password
+
+"""),
     cfg.StrOpt('vmware_server_password',
-               help=_('Password for authenticating with '
-                      'VMware ESX/VC server.'),
+               sample_default='vmware',
+               help="""
+Server password.
+
+This configuration option takes the password for authenticating with
+the VMware ESX/ESXi or vCenter Server. This option is required when
+using the VMware storage backend.
+
+Possible Values:
+    * Any string that is a password corresponding to the username
+      specified using the "vmware_server_username" option
+
+Related options:
+    * vmware_server_host
+    * vmware_server_username
+
+""",
                secret=True),
     cfg.IntOpt('vmware_api_retry_count',
                default=10,
-               help=_('Number of times VMware ESX/VC server API must be '
-                      'retried upon connection related issues.')),
+               min=1,
+               help="""
+The number of VMware API retries.
+
+This configuration option specifies the number of times the VMware
+ESX/VC server API must be retried upon connection related issues or
+server API call overload. It is not possible to specify 'retry
+forever'.
+
+Possible Values:
+    * Any positive integer value
+
+Related options:
+    * None
+
+"""),
     cfg.IntOpt('vmware_task_poll_interval',
                default=5,
-               help=_('The interval used for polling remote tasks '
-                      'invoked on VMware ESX/VC server.')),
+               min=1,
+               help="""
+Interval in seconds used for polling remote tasks invoked on VMware
+ESX/VC server.
+
+This configuration option takes in the sleep time in seconds for polling an
+on-going async task as part of the VMWare ESX/VC server API call.
+
+Possible Values:
+    * Any positive integer value
+
+Related options:
+    * None
+
+"""),
     cfg.StrOpt('vmware_store_image_dir',
                default=DEFAULT_STORE_IMAGE_DIR,
-               help=_('The name of the directory where the glance images '
-                      'will be stored in the VMware datastore.')),
+               help="""
+The directory where the glance images will be stored in the datastore.
+
+This configuration option specifies the path to the directory where the
+glance images will be stored in the VMware datastore. If this option
+is not set,  the default directory where the glance images are stored
+is openstack_glance.
+
+Possible Values:
+    * Any string that is a valid path to a directory
+
+Related options:
+    * None
+
+"""),
     cfg.BoolOpt('vmware_insecure',
                 default=False,
-                help=_('If true, the ESX/vCenter server certificate is not '
-                       'verified. If false, then the default CA truststore is '
-                       'used for verification. This option is ignored if '
-                       '"vmware_ca_file" is set.'),
-                deprecated_name='vmware_api_insecure'),
+                deprecated_name='vmware_api_insecure',
+                help="""
+Set verification of the ESX/vCenter server certificate.
+
+This configuration option takes a boolean value to determine
+whether or not to verify the ESX/vCenter server certificate. If this
+option is set to True, the ESX/vCenter server certificate is not
+verified. If this option is set to False, then the default CA
+truststore is used for verification.
+
+This option is ignored if the "vmware_ca_file" option is set. In that
+case, the ESX/vCenter server certificate will then be verified using
+the file specified using the "vmware_ca_file" option .
+
+Possible Values:
+    * True
+    * False
+
+Related options:
+    * vmware_ca_file
+
+"""),
     cfg.StrOpt('vmware_ca_file',
-               help=_('Specify a CA bundle file to use in verifying the '
-                      'ESX/vCenter server certificate.')),
+               sample_default='/etc/ssl/certs/ca-certificates.crt',
+               help="""
+Absolute path to the CA bundle file.
+
+This configuration option enables the operator to use a custom
+Cerificate Authority File to verify the ESX/vCenter certificate.
+
+If this option is set, the "vmware_insecure" option will be ignored
+and the CA file specified will be used to authenticate the ESX/vCenter
+server certificate and establish a secure connection to the server.
+
+Possible Values:
+    * Any string that is a valid absolute path to a CA file
+
+Related options:
+    * vmware_insecure
+
+"""),
     cfg.MultiStrOpt(
         'vmware_datastores',
-        help=_(
-            'A list of datastores where the image can be stored. This option '
-            'may be specified multiple times for specifying multiple '
-            'datastores. The datastore name should be specified after its '
-            'datacenter path, seperated by ":". An optional weight may be '
-            'given after the datastore name, seperated again by ":". Thus, '
-            'the required format becomes '
-            '<datacenter_path>:<datastore_name>:<optional_weight>. When '
-            'adding an image, the datastore with highest weight will be '
-            'selected, unless there is not enough free space available in '
-            'cases where the image size is already known. If no weight is '
-            'given, it is assumed to be zero and the directory will be '
-            'considered for selection last. If multiple datastores have the '
-            'same weight, then the one with the most free space available is '
-            'selected.'))]
+        help="""
+The datastores where the image can be stored.
+
+This configuration option specifies the datastores where the image can
+be stored in the VMWare store backend. This option may be specified
+multiple times for specifying multiple datastores. The datastore name
+should be specified after its datacenter path, separated by ":". An
+optional weight may be given after the datastore name, separated again
+by ":" to specify the priority. Thus, the required format becomes
+<datacenter_path>:<datastore_name>:<optional_weight>.
+
+When adding an image, the datastore with highest weight will be
+selected, unless there is not enough free space available in cases
+where the image size is already known. If no weight is given, it is
+assumed to be zero and the directory will be considered for selection
+last. If multiple datastores have the same weight, then the one with
+the most free space available is selected.
+
+Possible Values:
+    * Any string of the format:
+      <datacenter_path>:<datastore_name>:<optional_weight>
+
+Related options:
+   * None
+
+""")]
 
 
 def http_response_iterator(conn, response, size):
@@ -132,16 +258,18 @@ def http_response_iterator(conn, response, size):
 
 class _Reader(object):
 
-    def __init__(self, data, verifier=None):
+    def __init__(self, data, hashing_algo, verifier=None):
         self._size = 0
         self.data = data
         self.checksum = hashlib.md5()
+        self.os_hash_value = hashlib.new(str(hashing_algo))
         self.verifier = verifier
 
     def read(self, size=None):
         result = self.data.read(size)
         self._size += len(result)
         self.checksum.update(result)
+        self.os_hash_value.update(result)
         if self.verifier:
             self.verifier.update(result)
         return result
@@ -158,10 +286,12 @@ class StoreLocation(location.StoreLocation):
     vsphere://server_host/folder/file_path?dcPath=dc_path&dsName=ds_name
     """
 
-    def __init__(self, store_specs, conf):
-        super(StoreLocation, self).__init__(store_specs, conf)
+    def __init__(self, store_specs, conf, backend_group=None):
+        super(StoreLocation, self).__init__(store_specs, conf,
+                                            backend_group=backend_group)
         self.datacenter_path = None
         self.datastore_name = None
+        self.backend_group = backend_group
 
     def process_specs(self):
         self.scheme = self.specs.get('scheme', STORE_SCHEME)
@@ -194,11 +324,7 @@ class StoreLocation(location.StoreLocation):
     #    return path.startswith(os.path.join(DS_URL_PREFIX, sdir))
 
     def parse_uri(self, uri):
-        if not uri.startswith('%s://' % STORE_SCHEME):
-            reason = (_("URI %(uri)s must start with %(scheme)s://") %
-                      {'uri': uri, 'scheme': STORE_SCHEME})
-            LOG.info(reason)
-            raise exceptions.BadStoreUri(message=reason)
+        self.validate_schemas(uri, valid_schemas=('%s://' % STORE_SCHEME,))
         (self.scheme, self.server_host,
          path, params, query, fragment) = urllib.parse.urlparse(uri)
         if not query:
@@ -237,8 +363,8 @@ class Store(glance_store.Store):
     OPTIONS = _VMWARE_OPTS
     WRITE_CHUNKSIZE = units.Mi
 
-    def __init__(self, conf):
-        super(Store, self).__init__(conf)
+    def __init__(self, conf, backend=None):
+        super(Store, self).__init__(conf, backend=backend)
         self.datastores = {}
 
     def reset_session(self):
@@ -253,13 +379,18 @@ class Store(glance_store.Store):
         return (STORE_SCHEME,)
 
     def _sanity_check(self):
-        if self.conf.glance_store.vmware_api_retry_count <= 0:
+        if self.backend_group:
+            store_conf = getattr(self.conf, self.backend_group)
+        else:
+            store_conf = self.conf.glance_store
+
+        if store_conf.vmware_api_retry_count <= 0:
             msg = _('vmware_api_retry_count should be greater than zero')
             LOG.error(msg)
             raise exceptions.BadStoreConfiguration(
                 store_name='vmware_datastore', reason=msg)
 
-        if self.conf.glance_store.vmware_task_poll_interval <= 0:
+        if store_conf.vmware_task_poll_interval <= 0:
             msg = _('vmware_task_poll_interval should be greater than zero')
             LOG.error(msg)
             raise exceptions.BadStoreConfiguration(
@@ -271,10 +402,16 @@ class Store(glance_store.Store):
         self.server_host = self._option_get('vmware_server_host')
         self.server_username = self._option_get('vmware_server_username')
         self.server_password = self._option_get('vmware_server_password')
-        self.api_retry_count = self.conf.glance_store.vmware_api_retry_count
-        self.tpoll_interval = self.conf.glance_store.vmware_task_poll_interval
-        self.ca_file = self.conf.glance_store.vmware_ca_file
-        self.api_insecure = self.conf.glance_store.vmware_insecure
+
+        if self.backend_group:
+            store_conf = getattr(self.conf, self.backend_group)
+        else:
+            store_conf = self.conf.glance_store
+
+        self.api_retry_count = store_conf.vmware_api_retry_count
+        self.tpoll_interval = store_conf.vmware_task_poll_interval
+        self.ca_file = store_conf.vmware_ca_file
+        self.api_insecure = store_conf.vmware_insecure
         if api is None:
             msg = _("Missing dependencies: oslo_vmware")
             raise exceptions.BadStoreConfiguration(
@@ -370,11 +507,17 @@ class Store(glance_store.Store):
     def configure_add(self):
         datastores = self._option_get('vmware_datastores')
         self.datastores = self._build_datastore_weighted_map(datastores)
-        self.store_image_dir = self.conf.glance_store.vmware_store_image_dir
+
+        if self.backend_group:
+            store_conf = getattr(self.conf, self.backend_group)
+        else:
+            store_conf = self.conf.glance_store
+
+        self.store_image_dir = store_conf.vmware_store_image_dir
 
     def select_datastore(self, image_size):
         """Select a datastore with free space larger than image size."""
-        for k, v in sorted(six.iteritems(self.datastores), reverse=True):
+        for k, v in sorted(self.datastores.items(), reverse=True):
             max_ds = None
             max_fs = 0
             for ds in v:
@@ -391,7 +534,12 @@ class Store(glance_store.Store):
         raise exceptions.StorageFull()
 
     def _option_get(self, param):
-        result = getattr(self.conf.glance_store, param)
+        if self.backend_group:
+            store_conf = getattr(self.conf, self.backend_group)
+        else:
+            store_conf = self.conf.glance_store
+
+        result = getattr(store_conf, param)
         if not result:
             reason = (_("Could not find %(param)s in configuration "
                         "options.") % {'param': param})
@@ -408,8 +556,9 @@ class Store(glance_store.Store):
             cookie = list(vim_cookies)[0]
             return cookie.name + '=' + cookie.value
 
+    @glance_store.driver.back_compat_add
     @capabilities.check
-    def add(self, image_id, image_file, image_size, context=None,
+    def add(self, image_id, image_file, image_size, hashing_algo, context=None,
             verifier=None):
         """Stores an image file with supplied identifier to the backend
         storage system and returns a tuple containing information
@@ -418,20 +567,24 @@ class Store(glance_store.Store):
         :param image_id: The opaque image identifier
         :param image_file: The image data to write, as a file-like object
         :param image_size: The size of the image data to write, in bytes
+        :param hashing_algo: A hashlib algorithm identifier (string)
+        :param context: A context object
         :param verifier: An object used to verify signatures for images
-        :retval tuple of URL in backing store, bytes written, checksum
-                and a dictionary with storage system specific information
-        :raises: `glance.common.exceptions.Duplicate` if the image already
-                existed
-                `glance.common.exceptions.UnexpectedStatus` if the upload
-                request returned an unexpected status. The expected responses
-                are 201 Created and 200 OK.
+
+        :returns: tuple of: (1) URL in backing store, (2) bytes written,
+                  (3) checksum, (4) multihash value, and (5) a dictionary
+                  with storage system specific information
+        :raises: `glance_store.exceptions.Duplicate` if the image already
+                 exists
+        :raises: `glance.common.exceptions.UnexpectedStatus` if the upload
+                 request returned an unexpected status. The expected responses
+                 are 201 Created and 200 OK.
         """
         ds = self.select_datastore(image_size)
-        image_file = _Reader(image_file, verifier)
+        image_file = _Reader(image_file, hashing_algo, verifier)
         headers = {}
         if image_size > 0:
-            headers.update({'Content-Length': image_size})
+            headers.update({'Content-Length': six.text_type(image_size)})
             data = image_file
         else:
             data = utils.chunkiter(image_file, CHUNKSIZE)
@@ -440,7 +593,8 @@ class Store(glance_store.Store):
                              'image_dir': self.store_image_dir,
                              'datacenter_path': ds.datacenter.path,
                              'datastore_name': ds.name,
-                             'image_id': image_id}, self.conf)
+                             'image_id': image_id}, self.conf,
+                            backend_group=self.backend_group)
         # NOTE(arnaud): use a decorator when the config is not tied to self
         cookie = self._build_vim_cookie_header(True)
         headers = dict(headers)
@@ -487,8 +641,15 @@ class Store(glance_store.Store):
             LOG.error(msg)
             raise exceptions.BackendException(msg)
 
-        return (loc.get_uri(), image_file.size,
-                image_file.checksum.hexdigest(), {})
+        metadata = {}
+        if self.backend_group:
+            metadata['backend'] = u"%s" % self.backend_group
+
+        return (loc.get_uri(),
+                image_file.size,
+                image_file.checksum.hexdigest(),
+                image_file.os_hash_value.hexdigest(),
+                metadata)
 
     @capabilities.check
     def get(self, location, offset=0, chunk_size=None, context=None):
@@ -638,7 +799,8 @@ class Store(glance_store.Store):
                                               self.conf,
                                               uri=vsphere_url,
                                               image_id=image_id,
-                                              store_specs=store_specs)
+                                              store_specs=store_specs,
+                                              backend=self.backend_group)
 
 
 def new_session(insecure=False, ca_file=None, total_retries=None):
