@@ -1,4 +1,5 @@
 # Copyright 2010-2011 OpenStack Foundation
+# Copyright 2018 Verizon Wireless
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -13,6 +14,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import hashlib
 import logging
 
 from oslo_config import cfg
@@ -23,32 +25,140 @@ from stevedore import extension
 
 from glance_store import capabilities
 from glance_store import exceptions
-from glance_store import i18n
+from glance_store.i18n import _
 from glance_store import location
 
-
 CONF = cfg.CONF
 LOG = logging.getLogger(__name__)
 
-_ = i18n._
-
 _STORE_OPTS = [
-    cfg.ListOpt('stores', default=['file', 'http'],
-                help=_("List of stores enabled. Valid stores are: "
-                       "cinder, file, http, rbd, sheepdog, swift, "
-                       "s3, vsphere")),
-    cfg.StrOpt('default_store', default='file',
-               help=_("Default scheme to use to store image data. The "
-                      "scheme must be registered by one of the stores "
-                      "defined by the 'stores' config option.")),
-    cfg.IntOpt('store_capabilities_update_min_interval', default=0,
-               help=_("Minimum interval seconds to execute updating "
-                      "dynamic storage capabilities based on backend "
-                      "status then. It's not a periodic routine, the "
-                      "update logic will be executed only when interval "
-                      "seconds elapsed and an operation of store has "
-                      "triggered. The feature will be enabled only when "
-                      "the option value greater then zero."))
+    cfg.ListOpt('stores',
+                default=['file', 'http'],
+                deprecated_for_removal=True,
+                deprecated_since='Rocky',
+                deprecated_reason="""
+This option is deprecated against new config option
+``enabled_backends`` which helps to configure multiple backend stores
+of different schemes.
+
+This option is scheduled for removal in the Stein development
+cycle.
+""",
+                help="""
+List of enabled Glance stores.
+
+Register the storage backends to use for storing disk images
+as a comma separated list. The default stores enabled for
+storing disk images with Glance are ``file`` and ``http``.
+
+Possible values:
+    * A comma separated list that could include:
+        * file
+        * http
+        * swift
+        * rbd
+        * sheepdog
+        * cinder
+        * vmware
+
+Related Options:
+    * default_store
+
+"""),
+    cfg.StrOpt('default_store',
+               default='file',
+               choices=('file', 'filesystem', 'http', 'https', 'swift',
+                        'swift+http', 'swift+https', 'swift+config', 'rbd',
+                        'sheepdog', 'cinder', 'vsphere'),
+               deprecated_for_removal=True,
+               deprecated_since='Rocky',
+               deprecated_reason="""
+This option is deprecated against new config option
+``default_backend`` which acts similar to ``default_store`` config
+option.
+
+This option is scheduled for removal in the Stein development
+cycle.
+""",
+               help="""
+The default scheme to use for storing images.
+
+Provide a string value representing the default scheme to use for
+storing images. If not set, Glance uses ``file`` as the default
+scheme to store images with the ``file`` store.
+
+NOTE: The value given for this configuration option must be a valid
+scheme for a store registered with the ``stores`` configuration
+option.
+
+Possible values:
+    * file
+    * filesystem
+    * http
+    * https
+    * swift
+    * swift+http
+    * swift+https
+    * swift+config
+    * rbd
+    * sheepdog
+    * cinder
+    * vsphere
+
+Related Options:
+    * stores
+
+"""),
+    cfg.IntOpt('store_capabilities_update_min_interval',
+               default=0,
+               min=0,
+               deprecated_for_removal=True,
+               deprecated_since='Rocky',
+               deprecated_reason="""
+This option configures a stub method that has not been implemented
+for any existing store drivers.  Hence it is non-operational, and
+giving it a value does absolutely nothing.
+
+This option is scheduled for removal early in the Stein development
+cycle.
+""",
+               help="""
+Minimum interval in seconds to execute updating dynamic storage
+capabilities based on current backend status.
+
+Provide an integer value representing time in seconds to set the
+minimum interval before an update of dynamic storage capabilities
+for a storage backend can be attempted. Setting
+``store_capabilities_update_min_interval`` does not mean updates
+occur periodically based on the set interval. Rather, the update
+is performed at the elapse of this interval set, if an operation
+of the store is triggered.
+
+By default, this option is set to zero and is disabled. Provide an
+integer value greater than zero to enable this option.
+
+NOTE 1: For more information on store capabilities and their updates,
+please visit: https://specs.openstack.org/openstack/glance-specs/\
+specs/kilo/store-capabilities.html
+
+For more information on setting up a particular store in your
+deployment and help with the usage of this feature, please contact
+the storage driver maintainers listed here:
+https://docs.openstack.org/glance_store/latest/user/drivers.html
+
+NOTE 2: The dynamic store update capability described above is not
+implemented by any current store drivers.  Thus, this option DOES
+NOT DO ANYTHING (and it never has).  It is DEPRECATED and scheduled
+for removal early in the Stein development cycle.
+
+Possible values:
+    * Zero
+    * Positive integer
+
+Related Options:
+    * None
+
+"""),
 ]
 
 _STORE_CFG_GROUP = 'glance_store'
@@ -58,7 +168,13 @@ def _list_opts():
     driver_opts = []
     mgr = extension.ExtensionManager('glance_store.drivers')
     # NOTE(zhiyan): Handle available drivers entry_points provided
-    drivers = [ext.name for ext in mgr]
+    # NOTE(nikhil): Return a sorted list of drivers to ensure that the sample
+    # configuration files generated by oslo config generator retain the order
+    # in which the config opts appear across different runs. If this order of
+    # config opts is not preserved, some downstream packagers may see a long
+    # diff of the changes though not relevant as only order has changed. See
+    # some more details at bug 1619487.
+    drivers = sorted([ext.name for ext in mgr])
     handled_drivers = []  # Used to handle backwards-compatible entries
     for store_entry in drivers:
         driver_cls = _load_store(None, store_entry, False)
@@ -322,6 +438,25 @@ def check_location_metadata(val, key=''):
                                           % dict(key=key, type=type(val)))
 
 
+def _check_metadata(store, metadata):
+    if not isinstance(metadata, dict):
+        msg = (_("The storage driver %(driver)s returned invalid "
+                 " metadata %(metadata)s. This must be a dictionary type")
+               % dict(driver=str(store), metadata=str(metadata)))
+        LOG.error(msg)
+        raise exceptions.BackendException(msg)
+    try:
+        check_location_metadata(metadata)
+    except exceptions.BackendException as e:
+        e_msg = (_("A bad metadata structure was returned from the "
+                   "%(driver)s storage driver: %(metadata)s.  %(e)s.") %
+                 dict(driver=encodeutils.exception_to_unicode(store),
+                      metadata=encodeutils.exception_to_unicode(metadata),
+                      e=encodeutils.exception_to_unicode(e)))
+        LOG.error(e_msg)
+        raise exceptions.BackendException(e_msg)
+
+
 def store_add_to_backend(image_id, data, size, store, context=None,
                          verifier=None):
     """
@@ -332,6 +467,8 @@ def store_add_to_backend(image_id, data, size, store, context=None,
     :param data: The data to be stored
     :param size: The length of the data in bytes
     :param store: The store to which the data is being added
+    :param context: The request context
+    :param verifier: An object used to verify signatures for images
     :return: The url location of the file,
              the size amount of data,
              the checksum of the data
@@ -343,25 +480,49 @@ def store_add_to_backend(image_id, data, size, store, context=None,
                                                      context=context,
                                                      verifier=verifier)
     if metadata is not None:
-        if not isinstance(metadata, dict):
-            msg = (_("The storage driver %(driver)s returned invalid "
-                     " metadata %(metadata)s. This must be a dictionary type")
-                   % dict(driver=str(store), metadata=str(metadata)))
-            LOG.error(msg)
-            raise exceptions.BackendException(msg)
-        try:
-            check_location_metadata(metadata)
-        except exceptions.BackendException as e:
-            e_msg = (_("A bad metadata structure was returned from the "
-                       "%(driver)s storage driver: %(metadata)s.  %(e)s.") %
-                     dict(driver=encodeutils.exception_to_unicode(store),
-                          metadata=encodeutils.exception_to_unicode(metadata),
-                          e=encodeutils.exception_to_unicode(e)))
-            LOG.error(e_msg)
-            raise exceptions.BackendException(e_msg)
+        _check_metadata(store, metadata)
+
     return (location, size, checksum, metadata)
 
 
+def store_add_to_backend_with_multihash(
+        image_id, data, size, hashing_algo, store,
+        context=None, verifier=None):
+    """
+    A wrapper around a call to each store's add() method that requires
+    a hashing_algo identifier and returns a 5-tuple including the
+    "multihash" computed using the specified hashing_algo.  (This
+    is an enhanced version of store_add_to_backend(), which is left
+    as-is for backward compatibility.)
+
+    :param image_id:  The image add to which data is added
+    :param data: The data to be stored
+    :param size: The length of the data in bytes
+    :param store: The store to which the data is being added
+    :param hashing_algo: A hashlib algorithm identifier (string)
+    :param context: The request context
+    :param verifier: An object used to verify signatures for images
+    :return: The url location of the file,
+             the size amount of data,
+             the checksum of the data,
+             the multihash of the data,
+             the storage system's metadata dictionary for the location
+    :raises: ``glance_store.exceptions.BackendException``
+             ``glance_store.exceptions.UnknownHashingAlgo``
+    """
+
+    if hashing_algo not in hashlib.algorithms_available:
+        raise exceptions.UnknownHashingAlgo(algo=hashing_algo)
+
+    (location, size, checksum, multihash, metadata) = store.add(
+        image_id, data, size, hashing_algo, context=context, verifier=verifier)
+
+    if metadata is not None:
+        _check_metadata(store, metadata)
+
+    return (location, size, checksum, multihash, metadata)
+
+
 def add_to_backend(conf, image_id, data, size, scheme=None, context=None,
                    verifier=None):
     if scheme is None:
@@ -371,6 +532,15 @@ def add_to_backend(conf, image_id, data, size, scheme=None, context=None,
                                 verifier)
 
 
+def add_to_backend_with_multihash(conf, image_id, data, size, hashing_algo,
+                                  scheme=None, context=None, verifier=None):
+    if scheme is None:
+        scheme = conf['glance_store']['default_store']
+    store = get_store_from_scheme(scheme)
+    return store_add_to_backend_with_multihash(
+        image_id, data, size, hashing_algo, store, context, verifier)
+
+
 def set_acls(location_uri, public=False, read_tenants=[],
              write_tenants=None, context=None):
 
