@@ -43,11 +43,13 @@ from oslo_config import cfg
 from six.moves import urllib
 
 from glance_store import exceptions
+from glance_store.i18n import _
 
 CONF = cfg.CONF
 LOG = logging.getLogger(__name__)
 
 SCHEME_TO_CLS_MAP = {}
+SCHEME_TO_CLS_BACKEND_MAP = {}
 
 
 def get_location_from_uri(uri, conf=CONF):
@@ -61,7 +63,7 @@ def get_location_from_uri(uri, conf=CONF):
 
     Example URIs:
         https://user:pass@example.com:80/images/some-id
-        http://images.oracle.com/123456
+        http://example.com/123456
         swift://example.com/container/obj-id
         swift://user:account:pass@authurl.com/container/obj-id
         swift+http://user:account:pass@authurl.com/container/obj-id
@@ -76,6 +78,58 @@ def get_location_from_uri(uri, conf=CONF):
                     conf, uri=uri)
 
 
+def get_location_from_uri_and_backend(uri, backend, conf=CONF):
+    """Extract backend location from a URI.
+
+    Given a URI, return a Location object that has had an appropriate
+    store parse the URI.
+
+    :param uri: A URI that could come from the end-user in the Location
+                attribute/header.
+    :param backend: A backend name for the store.
+    :param conf: The global configuration.
+
+    Example URIs:
+        https://user:pass@example.com:80/images/some-id
+        http://example.com/123456
+        swift://example.com/container/obj-id
+        swift://user:account:pass@authurl.com/container/obj-id
+        swift+http://user:account:pass@authurl.com/container/obj-id
+        file:///var/lib/glance/images/1
+        cinder://volume-id
+    """
+
+    pieces = urllib.parse.urlparse(uri)
+
+    if pieces.scheme not in SCHEME_TO_CLS_BACKEND_MAP.keys():
+        raise exceptions.UnknownScheme(scheme=pieces.scheme)
+    try:
+        scheme_info = SCHEME_TO_CLS_BACKEND_MAP[pieces.scheme][backend]
+    except KeyError:
+        raise exceptions.UnknownScheme(scheme=backend)
+
+    return Location(pieces.scheme, scheme_info['location_class'],
+                    conf, uri=uri, backend=backend)
+
+
+def register_scheme_backend_map(scheme_map):
+    """Registers a mapping between a scheme and a backend.
+
+    Given a mapping of 'scheme' to store_name, adds the mapping to the
+    known list of schemes.
+
+    This function overrides existing stores.
+    """
+
+    for (k, v) in scheme_map.items():
+        if k not in SCHEME_TO_CLS_BACKEND_MAP:
+            SCHEME_TO_CLS_BACKEND_MAP[k] = {}
+
+        LOG.debug("Registering scheme %s with %s", k, v)
+        for key, value in v.items():
+            SCHEME_TO_CLS_BACKEND_MAP[k][key] = value
+
+
 def register_scheme_map(scheme_map):
     """
     Given a mapping of 'scheme' to store_name, adds the mapping to the
@@ -96,7 +150,7 @@ class Location(object):
     """
 
     def __init__(self, store_name, store_location_class, conf,
-                 uri=None, image_id=None, store_specs=None):
+                 uri=None, image_id=None, store_specs=None, backend=None):
         """
         Create a new Location object.
 
@@ -109,12 +163,15 @@ class Location(object):
         :param store_specs: Dictionary of information about the location
                             of the image that is dependent on the backend
                             store
+        :param backend: Name of store backend
         """
         self.store_name = store_name
         self.image_id = image_id
         self.store_specs = store_specs or {}
         self.conf = conf
-        self.store_location = store_location_class(self.store_specs, conf)
+        self.backend_group = backend
+        self.store_location = store_location_class(
+            self.store_specs, conf, backend_group=backend)
         if uri:
             self.store_location.parse_uri(uri)
 
@@ -135,9 +192,10 @@ class StoreLocation(object):
     Base class that must be implemented by each store
     """
 
-    def __init__(self, store_specs, conf):
+    def __init__(self, store_specs, conf, backend_group=None):
         self.conf = conf
         self.specs = store_specs
+        self.backend_group = backend_group
         if self.specs:
             self.process_specs()
 
@@ -167,3 +225,17 @@ class StoreLocation(object):
         """
         raise NotImplementedError("StoreLocation subclass must implement "
                                   "parse_uri()")
+
+    @staticmethod
+    def validate_schemas(uri, valid_schemas):
+        """check if uri scheme is one of valid_schemas
+        generate exception otherwise
+        """
+        for valid_schema in valid_schemas:
+            if uri.startswith(valid_schema):
+                return
+
+        reason = _("Location URI must start with one of the following "
+                   "schemas: %s") % ', '.join(valid_schemas)
+        LOG.warning(reason)
+        raise exceptions.BadStoreUri(message=reason)
